aboutsummaryrefslogtreecommitdiff
path: root/src/cmd/compile/internal/ssa/rewriteARM64.go
diff options
context:
space:
mode:
authorJosh Bleecher Snyder <josharian@gmail.com>2020-02-16 17:00:52 -0800
committerJosh Bleecher Snyder <josharian@gmail.com>2020-02-26 19:30:21 +0000
commiteb5cd0fb40f24d8480fd549ad36c34ed001ccf3d (patch)
treed86f517eed791b03bd87f3afd72442b37818eb99 /src/cmd/compile/internal/ssa/rewriteARM64.go
parent390c096ee99728b8e1b0218a775a39651baf057e (diff)
downloadgo-eb5cd0fb40f24d8480fd549ad36c34ed001ccf3d.tar.gz
go-eb5cd0fb40f24d8480fd549ad36c34ed001ccf3d.zip
cmd/compile: mark Lsyms as readonly earlier
The SSA backend has rules to read the contents of readonly Lsyms. However, this rule was failing to trigger for many readonly Lsyms. This is because the readonly attribute that was set on the Node.Name was not propagated to its Lsym until the dump globals phase, after SSA runs. To work around this phase ordering problem, introduce Node.SetReadonly, which sets Node.Name.Readonly and also configures the Lsym enough that SSA can use it. This change also fixes a latent problem in the rewrite rule function, namely that reads past the end of lsym.P were treated as entirely zero, instead of merely requiring padding with trailing zeros. This change also adds an amd64 rule needed to fully optimize the results of this change. It would be better not to need this, but the zero extension that should handle this for us gets optimized away too soon (see #36897 for a similar problem). I have not investigated whether other platforms also need new rules to take full advantage of the new optimizations. Compiled code for (interface{})(true) on amd64 goes from: LEAQ type.bool(SB), AX MOVBLZX ""..stmp_0(SB), BX LEAQ runtime.staticbytes(SB), CX ADDQ CX, BX to LEAQ type.bool(SB), AX LEAQ runtime.staticbytes+1(SB), BX Prior to this change, the readonly symbol rewrite rules fired a total of 884 times during make.bash. Afterwards they fire 1807 times. file before after Δ % cgo 4827832 4823736 -4096 -0.085% compile 24907768 24895656 -12112 -0.049% fix 3376952 3368760 -8192 -0.243% pprof 14751700 14747604 -4096 -0.028% total 120343528 120315032 -28496 -0.024% Change-Id: I59ea52138276c37840f69e30fb109fd376d579ec Reviewed-on: https://go-review.googlesource.com/c/go/+/220499 Run-TryBot: Josh Bleecher Snyder <josharian@gmail.com> TryBot-Result: Gobot Gobot <gobot@golang.org> Reviewed-by: Keith Randall <khr@golang.org>
Diffstat (limited to 'src/cmd/compile/internal/ssa/rewriteARM64.go')
-rw-r--r--src/cmd/compile/internal/ssa/rewriteARM64.go12
1 files changed, 6 insertions, 6 deletions
diff --git a/src/cmd/compile/internal/ssa/rewriteARM64.go b/src/cmd/compile/internal/ssa/rewriteARM64.go
index a4c66cee1a..3bb86f10bc 100644
--- a/src/cmd/compile/internal/ssa/rewriteARM64.go
+++ b/src/cmd/compile/internal/ssa/rewriteARM64.go
@@ -9775,7 +9775,7 @@ func rewriteValueARM64_OpARM64MOVDload(v *Value) bool {
}
// match: (MOVDload [off] {sym} (SB) _)
// cond: symIsRO(sym)
// result: (MOVDconst [int64(read64(sym, off, config.BigEndian))])
for {
off := v.AuxInt
sym := v.Aux
@@ -9783,7 +9783,7 @@ func rewriteValueARM64_OpARM64MOVDload(v *Value) bool {
break
}
v.reset(OpARM64MOVDconst)
- v.AuxInt = int64(read64(sym, off, config.BigEndian))
+ v.AuxInt = int64(read64(sym, off, config.ctxt.Arch.ByteOrder))
return true
}
return false
@@ -10599,7 +10599,7 @@ func rewriteValueARM64_OpARM64MOVHUload(v *Value) bool {
}
// match: (MOVHUload [off] {sym} (SB) _)
// cond: symIsRO(sym)
// result: (MOVDconst [int64(read16(sym, off, config.BigEndian))])
for {
off := v.AuxInt
sym := v.Aux
@@ -10607,7 +10607,7 @@ func rewriteValueARM64_OpARM64MOVHUload(v *Value) bool {
break
}
v.reset(OpARM64MOVDconst)
- v.AuxInt = int64(read16(sym, off, config.BigEndian))
+ v.AuxInt = int64(read16(sym, off, config.ctxt.Arch.ByteOrder))
return true
}
return false
@@ -12798,7 +12798,7 @@ func rewriteValueARM64_OpARM64MOVWUload(v *Value) bool {
}
// match: (MOVWUload [off] {sym} (SB) _)
// cond: symIsRO(sym)
// result: (MOVDconst [int64(read32(sym, off, config.BigEndian))])
for {
off := v.AuxInt
sym := v.Aux
@@ -12806,7 +12806,7 @@ func rewriteValueARM64_OpARM64MOVWUload(v *Value) bool {
break
}
v.reset(OpARM64MOVDconst)
- v.AuxInt = int64(read32(sym, off, config.BigEndian))
+ v.AuxInt = int64(read32(sym, off, config.ctxt.Arch.ByteOrder))
return true
}
return false
@@ -9775,7 +9775,7 @@ func rewriteValueARM64_OpARM64MOVDload(v *Value) bool {
// match: (MOVDload [off] {sym} (SB) _)
// cond: symIsRO(sym)
- // result: (MOVDconst [int64(read64(sym, off, config.BigEndian))])
+ // result: (MOVDconst [int64(read64(sym, off, config.ctxt.Arch.ByteOrder))])
for {
off := v.AuxInt
@@ -10599,7 +10599,7 @@ func rewriteValueARM64_OpARM64MOVHUload(v *Value) bool {
// match: (MOVHUload [off] {sym} (SB) _)
// cond: symIsRO(sym)
- // result: (MOVDconst [int64(read16(sym, off, config.BigEndian))])
+ // result: (MOVDconst [int64(read16(sym, off, config.ctxt.Arch.ByteOrder))])
for {
off := v.AuxInt
@@ -12798,7 +12798,7 @@ func rewriteValueARM64_OpARM64MOVWUload(v *Value) bool {
// match: (MOVWUload [off] {sym} (SB) _)
// cond: symIsRO(sym)
- // result: (MOVDconst [int64(read32(sym, off, config.BigEndian))])
+ // result: (MOVDconst [int64(read32(sym, off, config.ctxt.Arch.ByteOrder))])
for {
off := v.AuxInt